@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2007 Vreixo Formoso
- * Copyright (c) 2009 Thomas Schmitt
+ * Copyright (c) 2009 - 2013 Thomas Schmitt
  *
  * This file is part of the libisofs project; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License version 2 
@@ -8,6 +8,10 @@
  * See COPYING file for details.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include "libisofs.h"
 #include "image.h"
 #include "node.h"
@@ -28,11 +32,11 @@
  * @param image
  *     Location where the image pointer will be stored.
  * @return
- *     1 sucess, < 0 error
+ *     1 success, < 0 error
  */
 int iso_image_new(const char *name, IsoImage **image)
 {
-    int res;
+    int res, i;
     IsoImage *img;
 
     if (image == NULL) {
@@ -76,18 +80,22 @@ int iso_image_new(const char *name, IsoImage **image)
     }
     img->system_area_data = NULL;
     img->system_area_options = 0;
+    img->num_mips_boot_files = 0;
+    for (i = 0; i < 15; i++)
+         img->mips_boot_file_paths[i] = NULL;
+    img->sparc_core_node = NULL;
     img->builder_ignore_acl = 1;
     img->builder_ignore_ea = 1;
     img->inode_counter = 0;
     img->used_inodes = NULL;
     img->used_inodes_start = 0;
-
-#ifdef Libisofs_with_checksumS
     img->checksum_start_lba = 0;
     img->checksum_end_lba = 0;
     img->checksum_idx_count = 0;
     img->checksum_array = NULL;
-#endif
+    img->generator_is_running = 0;
+    for (i = 0; i < ISO_HFSPLUS_BLESS_MAX; i++)
+        img->hfsplus_blessed[i] = NULL;
 
     *image = img;
     return ISO_SUCCESS;
@@ -102,30 +110,35 @@ void iso_image_ref(IsoImage *image)
 }
 
 /**
- * Decrements the reference couting of the given image.
+ * Decrements the reference counting of the given image.
  * If it reaches 0, the image is free, together with its tree nodes (whether
  * their refcount reach 0 too, of course).
  */
 void iso_image_unref(IsoImage *image)
 {
-    if (--image->refcount == 0) {
-        int nexcl;
+    int nexcl, i;
 
+    if (--image->refcount == 0) {
         /* we need to free the image */
+
         if (image->user_data_free != NULL) {
             /* free attached data */
             image->user_data_free(image->user_data);
         }
-
         for (nexcl = 0; nexcl < image->nexcludes; ++nexcl) {
             free(image->excludes[nexcl]);
         }
         free(image->excludes);
-
+        for (i = 0; i < ISO_HFSPLUS_BLESS_MAX; i++)
+            if (image->hfsplus_blessed[i] != NULL)
+                iso_node_unref(image->hfsplus_blessed[i]);
         iso_node_unref((IsoNode*)image->root);
         iso_node_builder_unref(image->builder);
         iso_filesystem_unref(image->fs);
         el_torito_boot_catalog_free(image->bootcat);
+        iso_image_give_up_mips_boot(image, 0);
+        if (image->sparc_core_node != NULL)
+            iso_node_unref((IsoNode *) image->sparc_core_node);
         free(image->volset_id);
         free(image->volume_id);
         free(image->publisher_id);
@@ -135,8 +148,14 @@ void iso_image_unref(IsoImage *image)
         free(image->copyright_file_id);
         free(image->abstract_file_id);
         free(image->biblio_file_id);
+        free(image->creation_time);
+        free(image->modification_time);
+        free(image->expiration_time);
+        free(image->effective_time);
         if (image->used_inodes != NULL)
             free(image->used_inodes);
+        if (image->system_area_data != NULL)
+            free(image->system_area_data);
         iso_image_free_checksums(image, 0);
         free(image);
     }
@@ -145,17 +164,12 @@ void iso_image_unref(IsoImage *image)
 
 int iso_image_free_checksums(IsoImage *image, int flag)
 {
-#ifdef Libisofs_with_checksumS
-
     image->checksum_start_lba = 0;
     image->checksum_end_lba = 0;
     image->checksum_idx_count = 0;
     if (image->checksum_array != NULL)
         free(image->checksum_array);
     image->checksum_array = NULL;
-
-#endif /* Libisofs_with_checksumS */
-
     return 1;
 }
 
@@ -175,13 +189,13 @@ int iso_image_free_checksums(IsoImage *image, int flag)
  */
 int iso_image_attach_data(IsoImage *image, void *data, void (*give_up)(void*))
 {
-    if (image == NULL || (data != NULL && free == NULL)) {
+    if (image == NULL) {
         return ISO_NULL_POINTER;
     }
 
     if (image->user_data != NULL) {
         /* free previously attached data */
-        if (image->user_data_free) {
+        if (image->user_data_free != NULL) {
             image->user_data_free(image->user_data);
         }
         image->user_data = NULL;
@@ -328,6 +342,37 @@ const char *iso_image_get_biblio_file_id(const IsoImage *image)
     return image->biblio_file_id;
 }
 
+int iso_image_set_pvd_times(IsoImage *image,
+                            char *creation_time, char *modification_time,
+                            char *expiration_time, char *effective_time)
+{
+    if (creation_time == NULL || modification_time == NULL ||
+        expiration_time == NULL || effective_time == NULL)
+        return ISO_NULL_POINTER;
+    image->creation_time = strdup(creation_time);
+    image->modification_time = strdup(modification_time);
+    image->expiration_time = strdup(expiration_time);
+    image->effective_time = strdup(effective_time);
+    if (image->creation_time == NULL || image->modification_time == NULL ||
+        image->expiration_time == NULL || image->effective_time == NULL)
+        return ISO_OUT_OF_MEM;
+    return ISO_SUCCESS;
+}
+
+int iso_image_get_pvd_times(IsoImage *image,
+                            char **creation_time, char **modification_time,
+                            char **expiration_time, char **effective_time)
+{
+    if (image->creation_time == NULL || image->modification_time == NULL ||
+        image->expiration_time == NULL || image->effective_time == NULL)
+        return ISO_NULL_POINTER;
+    *creation_time = image->creation_time;
+    *modification_time = image->modification_time;
+    *expiration_time = image->expiration_time;
+    *effective_time = image->effective_time;
+    return ISO_SUCCESS;
+}
+
 int iso_image_get_msg_id(IsoImage *image)
 {
     return image->id;
@@ -592,39 +637,157 @@ ex:;
 int iso_image_get_session_md5(IsoImage *image, uint32_t *start_lba,
                               uint32_t *end_lba, char md5[16], int flag)
 {
-
-#ifdef Libisofs_with_checksumS
-
     if (image->checksum_array == NULL || image->checksum_idx_count < 1)
         return 0;
     *start_lba = image->checksum_start_lba;
     *end_lba = image->checksum_end_lba;
     memcpy(md5, image->checksum_array, 16);
     return ISO_SUCCESS;
-    
-#else
-
-    return 0;
-    
-#endif /* ! Libisofs_with_checksumS */
-    
 }
 
 int iso_image_set_checksums(IsoImage *image, char *checksum_array,
                             uint32_t start_lba, uint32_t end_lba,
                             uint32_t idx_count, int flag)
 {
-
-#ifdef Libisofs_with_checksumS
-
     iso_image_free_checksums(image, 0);
     image->checksum_array = checksum_array;
     image->checksum_start_lba = start_lba;
     image->checksum_end_lba = end_lba;
     image->checksum_idx_count = idx_count;
-
-#endif /* Libisofs_with_checksumS */
-
     return 1;
 }
 
+int iso_image_generator_is_running(IsoImage *image)
+{
+    return image->generator_is_running;
+}
+
+
+/* API */
+int iso_image_add_mips_boot_file(IsoImage *image, char *path, int flag)
+{
+    if (image->num_mips_boot_files >= 15)
+        return ISO_BOOT_TOO_MANY_MIPS;
+    image->mips_boot_file_paths[image->num_mips_boot_files] = strdup(path);
+    if (image->mips_boot_file_paths[image->num_mips_boot_files] == NULL)
+        return ISO_OUT_OF_MEM;
+    image->num_mips_boot_files++;
+    return ISO_SUCCESS;
+}
+
+/* API */
+int iso_image_get_mips_boot_files(IsoImage *image, char *paths[15], int flag)
+{
+    int i;
+
+    for (i = 0; i < image->num_mips_boot_files; i++)
+         paths[i] = image->mips_boot_file_paths[i];
+    for (; i < 15; i++)
+         paths[i] = NULL;
+    return image->num_mips_boot_files;
+}
+
+/* API */
+int iso_image_give_up_mips_boot(IsoImage *image, int flag)
+{
+    int i;
+
+    for (i = 0; i < image->num_mips_boot_files; i++)
+        if (image->mips_boot_file_paths[i] != NULL) {
+            free(image->mips_boot_file_paths[i]);
+            image->mips_boot_file_paths[i] = NULL;
+        }
+    image->num_mips_boot_files = 0;
+    return ISO_SUCCESS;
+}
+
+static void unset_blessing(IsoImage *img, unsigned int idx)
+{
+    if (img->hfsplus_blessed[idx] != NULL)
+        iso_node_unref(img->hfsplus_blessed[idx]);
+    img->hfsplus_blessed[idx] = NULL;
+}
+
+/* API */
+int iso_image_hfsplus_bless(IsoImage *img, enum IsoHfsplusBlessings blessing,
+                            IsoNode *node, int flag)
+{
+    unsigned int i, ok = 0;
+
+    if (flag & 2) {
+        /* Delete any blessing */
+        for (i = 0; i < ISO_HFSPLUS_BLESS_MAX; i++) {
+            if (img->hfsplus_blessed[i] == node || node == NULL) {
+                unset_blessing(img, i);
+                ok = 1;
+            }
+        }
+        return ok;
+    }
+    if (blessing == ISO_HFSPLUS_BLESS_MAX)
+        return ISO_WRONG_ARG_VALUE;
+    if (flag & 1) {
+        /* Delete a particular blessing */
+        if (img->hfsplus_blessed[blessing] == node || node == NULL) {
+            unset_blessing(img, (unsigned int) blessing);
+            return 1;
+        }
+        return 0;
+    }
+
+    if (node == NULL) {
+        unset_blessing(img, (unsigned int) blessing);
+        return 1;
+    }
+        
+    /* No two hats on one node */
+    for (i = 0; i < ISO_HFSPLUS_BLESS_MAX && node != NULL; i++)
+        if (i != blessing && img->hfsplus_blessed[i] == node)
+            return 0;
+    /* Enforce correct file type */
+    if (blessing == ISO_HFSPLUS_BLESS_INTEL_BOOTFILE) {
+        if (node->type != LIBISO_FILE)
+            return 0;
+    } else {
+        if (node->type != LIBISO_DIR)
+            return 0;
+    }
+
+    unset_blessing(img, (unsigned int) blessing);
+    img->hfsplus_blessed[blessing] = node;
+    if (node != NULL)
+        iso_node_ref(node);
+    return 1;
+}
+
+
+/* API */
+int iso_image_hfsplus_get_blessed(IsoImage *img, IsoNode ***blessed_nodes,
+                                  int *bless_max, int flag)
+{
+    *blessed_nodes = img->hfsplus_blessed;
+    *bless_max = ISO_HFSPLUS_BLESS_MAX;
+    return 1;
+}
+
+
+/* API */
+int iso_image_set_sparc_core(IsoImage *img, IsoFile *sparc_core, int flag)
+{
+    if (img->sparc_core_node != NULL)
+        iso_node_unref((IsoNode *) img->sparc_core_node);
+    img->sparc_core_node = sparc_core;
+    if (sparc_core != NULL)
+        iso_node_ref((IsoNode *) sparc_core);
+    return 1;
+}
+
+
+/* API */
+int iso_image_get_sparc_core(IsoImage *img, IsoFile **sparc_core, int flag)
+{
+    *sparc_core = img->sparc_core_node;
+    return 1;
+}
+
+
